@@ -72,7 +72,7 @@ except NameError:
     from html import escape as html_escape
 
 try:
-    from typeutils import make_sentinel
+    from .typeutils import make_sentinel
     _MISSING = make_sentinel(var_name='_MISSING')
 except ImportError:
     _MISSING = object()
