@@ -2,8 +2,10 @@
 
 from boltons.funcutils import (copy_function,
                                total_ordering,
+                               format_invocation,
                                InstancePartial,
-                               CachedInstancePartial)
+                               CachedInstancePartial,
+                               noop)
 
 
 class Greeter(object):
@@ -33,6 +35,15 @@ def test_partials():
     assert g.cached_partial_greet() == 'Hello...'
     assert CachedInstancePartial(g.greet, excitement='s')() == 'Hellos'
 
+    g.native_greet = 'native reassigned'
+    assert g.native_greet == 'native reassigned'
+
+    g.partial_greet = 'partial reassigned'
+    assert g.partial_greet == 'partial reassigned'
+
+    g.cached_partial_greet = 'cached_partial reassigned'
+    assert g.cached_partial_greet == 'cached_partial reassigned'
+
 
 def test_copy_function():
     def callee():
@@ -61,3 +72,15 @@ def test_total_ordering():
     assert num < 5
     assert num >= 2
     assert num != 1
+
+
+def test_format_invocation():
+    assert format_invocation('d') == "d()"
+    assert format_invocation('f', ('a', 'b')) == "f('a', 'b')"
+    assert format_invocation('g', (), {'x': 'y'})  == "g(x='y')"
+    assert format_invocation('h', ('a', 'b'), {'x': 'y', 'z': 'zz'}) == "h('a', 'b', x='y', z='zz')"
+
+def test_noop():
+    assert noop() is None
+    assert noop(1, 2) is None
+    assert noop(a=1, b=2) is None
